use crate::ui::UIWidget;
use maud::{Markup, Render, html};

use super::space::ScreenValue;

#[allow(non_snake_case)]
pub fn Width<T: UIWidget + 'static>(size: ScreenValue, inner: T) -> WidthWidget {
    WidthWidget(Box::new(inner), size, 0)
}

#[allow(non_snake_case)]
pub fn MinWidth<T: UIWidget + 'static>(size: ScreenValue, inner: T) -> WidthWidget {
    WidthWidget(Box::new(inner), size, 1)
}

#[allow(non_snake_case)]
pub fn MaxWidth<T: UIWidget + 'static>(size: ScreenValue, inner: T) -> WidthWidget {
    WidthWidget(Box::new(inner), size, 2)
}

pub struct WidthWidget(Box<dyn UIWidget>, ScreenValue, u8);

impl Render for WidthWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for WidthWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        match self.2 {
            1 => {
                return vec![format!("min-w-{}", self.1.to_value())];
            }
            2 => {
                return vec![format!("max-w-{}", self.1.to_value())];
            }
            _ => {}
        }

        vec![format!("w-{}", self.1.to_value())]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
